@@ -104,9 +104,9 @@ QString TimeLimitText(size_type limit) {
 	const auto weeks = (days / 7);
 	const auto months = (days / 29);
 	return (months > 0)
-		? tr::lng_local_storage_limit_months(tr::now, lt_count, months)
+		? tr::lng_months(tr::now, lt_count, months)
 		: (limit > 0)
-		? tr::lng_local_storage_limit_weeks(tr::now, lt_count, weeks)
+		? tr::lng_weeks(tr::now, lt_count, weeks)
 		: tr::lng_local_storage_limit_never(tr::now);
 }
 
@@ -167,6 +167,7 @@ LocalStorageBox::Row::Row(
 	sizeText(data),
 	st::localStorageRowSize)
 , _clear(this, std::move(clear), st::localStorageClear) {
+	_clear->setTextTransform(Ui::RoundButton::TextTransform::NoTransform);
 	_clear->setVisible(data.count != 0);
 }
 
@@ -241,7 +242,7 @@ void LocalStorageBox::Row::paintEvent(QPaintEvent *e) {
 	if (!_progress || true) {
 		return;
 	}
-	Painter p(this);
+	auto p = QPainter(this);
 	const auto padding = st::localStorageRowPadding;
 	const auto height = st::localStorageRowHeight;
 	const auto bottom = height - padding.bottom() - _description->height();
@@ -594,12 +595,3 @@ void LocalStorageBox::save() {
 	_session->data().cache().updateSettings(update);
 	closeBox();
 }
-
-void LocalStorageBox::paintEvent(QPaintEvent *e) {
-	BoxContent::paintEvent(e);
-
-	Painter p(this);
-
-	p.setFont(st::boxTextFont);
-	p.setPen(st::windowFg);
-}
