@@ -28,7 +28,7 @@ namespace {
 constexpr auto kWaitingFastDuration = crl::time(200);
 constexpr auto kWaitingShowDuration = crl::time(500);
 constexpr auto kWaitingShowDelay = crl::time(500);
-constexpr auto kGoodThumbnailQuality = 87;
+constexpr auto kGoodThumbQuality = 87;
 
 } // namespace
 
@@ -62,12 +62,17 @@ Document::Document(
 , _radial(
 		[=] { waitingCallback(); },
 		st::defaultInfiniteRadialAnimation) {
-	_player.updates(
+	resubscribe();
+}
+
+void Document::resubscribe() {
+	_subscription = _player.updates(
 	) | rpl::start_with_next_error([=](Update &&update) {
 		handleUpdate(std::move(update));
 	}, [=](Streaming::Error &&error) {
 		handleError(std::move(error));
-	}, _player.lifetime());
+		resubscribe();
+	});
 }
 
 Player &Document::player() {
@@ -91,12 +96,8 @@ void Document::play(const PlaybackOptions &options) {
 }
 
 void Document::saveFrameToCover() {
-	auto request = Streaming::FrameRequest();
-	//request.radius = (_doc && _doc->isVideoMessage())
-	//	? ImageRoundRadius::Ellipse
-	//	: ImageRoundRadius::None;
 	_info.video.cover = _player.ready()
-		? _player.frame(request)
+		? _player.currentFrameImage()
 		: _info.video.cover;
 }
 
@@ -221,6 +222,7 @@ void Document::validateGoodThumbnail() {
 		|| _document->goodThumbnailChecked()) {
 		return;
 	}
+	const auto sticker = (_document->sticker() != nullptr);
 	const auto document = _document;
 	const auto information = _info.video;
 	const auto key = document->goodThumbnailCacheKey();
@@ -242,12 +244,15 @@ void Document::validateGoodThumbnail() {
 					Qt::IgnoreAspectRatio,
 					Qt::SmoothTransformation);
 			}
+			if (!sticker && information.alpha) {
+				result = Images::Opaque(std::move(result));
+			}
 			return result;
 		}();
 		auto bytes = QByteArray();
 		{
 			auto buffer = QBuffer(&bytes);
-			image.save(&buffer, "JPG", kGoodThumbnailQuality);
+			image.save(&buffer, sticker ? "WEBP" : "JPG", kGoodThumbQuality);
 		}
 		const auto length = bytes.size();
 		if (!length || length > Storage::kMaxFileInMemory) {
